-- Exit if oUF isn't loaded
if not oUF then return end

--[[
Examples:

	-- Texture
	self.AFK = self:CreateTexture(nil, "OVERLAY")
	self.AFK:SetWidth(10)
	self.AFK:SetHeight(10)
	self.AFK:SetPoint("BOTTOM", self, "TOP")
	self.AFK:SetTexture(1, 0, 0)
	
OR
	-- Text, with timer
	self.AFK = self:CreateFontString(nil, "OVERLAY")
	self.AFK:SetFont("Fonts\\ARIALN.TTF", 10, "")
	self.AFK:SetWidth(10)
	self.AFK:SetHeight(10)
	self.AFK:SetPoint("BOTTOM", self, "TOP")
	self.AFK.fontFormat = "|cFFFF0000AFK |cFFFFFFFF%s:%s"
	
Replace self.AFK with self.DND to get DND status.
If you want to combine texture and text,
make sure that the function self.XXX.SetText(self, text) exists.
--]]

local timer = { }
local objects = { }
local timedObjects = { }

local updateFrame = CreateFrame("Frame", nil, UIParent)
updateFrame:Hide()

local updateTimerText = function(self)
	if self.next > GetTime() then return end
	self.next = GetTime() + 1
	
	local start = timer[ UnitName(self:GetParent().unit) or "xxx" ]
	if not start then
		self:Hide()
		timedObjects[self] = nil
		return
	end
	
	local esp = GetTime() - start
	local min = math.floor(esp/60)
	local sec = math.floor(esp-(min*60))
	if sec < 10 then sec = "0"..sec end
	self:SetText(self.fontFormat:format(min, sec))
end

local OnUpdate = function()
	local num = 0
	for self,_ in pairs(timedObjects) do
		updateTimerText(self)
		num = num + 1
	end
	if num == 0 then updateFrame:Hide() end
end

local flagChangedSub = function(self, key, unit, show)
	if self[key] then
		if show then
			self[key]:Show()
			if self[key].SetText then
				timer[UnitName(unit)] = timer[UnitName(unit)] or GetTime()
				timedObjects[ self[key] ] = true
				self[key].next = 0
				updateFrame:Show()
				return true
			end
		else
			self[key]:Hide()
			if self[key].SetText then
				timedObjects[ self[key] ] = nil
				return false
			end
		end
	end
end

local onEvent = function(_, event, unit)
	for _,self in ipairs(objects) do
		if self.unit == unit then
			if flagChangedSub(self, "AFK", unit, UnitIsAFK(unit)) then
				-- AFK
				flagChangedSub(self, "DND", unit, false) -- Hide DND
			elseif flagChangedSub(self, "DND", unit, UnitIsDND(unit)) then
				-- DND
				flagChangedSub(self, "AFK", unit, false) -- Hide AFK
			else
				-- Reset timer
				timer[UnitName(unit)] = nil
			end
		end
	end
end

updateFrame:RegisterEvent("PLAYER_FLAGS_CHANGED")
updateFrame:SetScript("OnEvent", onEvent)
updateFrame:SetScript("OnUpdate", OnUpdate)

local flagChanged = function(self, event, unit)
	if self.unit ~= unit then return end
	flagChangedSub(self, "AFK", unit, UnitIsAFK(unit))
	flagChangedSub(self, "DND", unit, UnitIsDND(unit))
end

local hook = function(object)
	if not object.AFK and not object.DND then return end
	tinsert(objects, object)
	
	-- Hook onto unit name updates
	if object.UNIT_NAME_UPDATE then
		old = object.UNIT_NAME_UPDATE
		object.UNIT_NAME_UPDATE = function(...)
			old(...)
			onEvent(...)
		end
	end
end

-- Append to all existing frames
for _,object in pairs(oUF.objects) do hook(object) end

-- Append to frames created later on
oUF:RegisterInitCallback(hook)
